#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# 形式ニューロン、AND素子の学習

import random

# 重みとバイアス、この2つの値を学習によって変化させる
W = [ random.random() for _ in range(2) ] 
    # 重み×2、初期値をランダムに設定する
B = random.random() 
    # バイアス、初期値をランダムに設定する（0.0以上1.0未満）

LEARNING_RATE = 0.1      # 学習係数 - 適切な小さな値を設定する

for trial in range(100): # 100回訓練を繰り返す

    # 次の訓練データをランダムに用意する
    x = [random.randint(0,1) for _ in range(2)] 
        # ランダムな1,0データを2個生成
    y = x[0] and x[1]   # ANDの訓練データ
    # y = x[0] or x[1]   # ORの訓練データ
    
    # 論理素子（形式ニューロン）を動かす
    u = sum([ W[i] * x[i] for i in range(2) ]) + B
    result = 1 if u > 0 else 0
    
    if result != y: # 結果が間違っていた 
        if result == 0:    # 本来 1が出てほしいのに 0が出た
            if x[0] == 1:
                W[0] += LEARNING_RATE   # 重みを増やす
            if x[1] == 1:
                W[1] += LEARNING_RATE   # 重みを増やす
            B += LEARNING_RATE      # バイアスを増やす
        else:              # 本来 0が出てほしいのに 1が出た
            if x[0] == 1:
                W[0] -= LEARNING_RATE   # 重みを減らす
            if x[1] == 1:
                W[1] -= LEARNING_RATE   # 重みを減らす
            B -= LEARNING_RATE      # バイアスを減らす

# 学習結果を確認する
for x in [ (0,0), (0,1), (1,0), (1,1) ]:
    u = sum([ W[i] * x[i] for i in range(len(x)) ]) + B
    result = 1 if u > 0 else 0
    print( "{} = {}".format( x, result ) )

