#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# ロジスティック回帰
# 誤差にシグモイドの微分を組み入れる

import numpy as np

N_INPUT = 2     # 入力データは2次元
N_DATA = 100    # データ件数は100件

from sklearn.datasets import make_blobs
train_x, train_y = make_blobs(  # 人工的なテストデータを用意する
            n_samples = N_DATA,
            n_features = N_INPUT,
            cluster_std = 3.0,  # データの塊の標準偏差
            centers = 2 )       # データの塊の数

W = np.random.rand(N_INPUT) - 0.5   # 重みを乱数で初期化
B = np.random.rand() - 0.5  # バイアスを乱数で初期化

LEARNING_RATE = 0.01    # 学習係数・ここをうまく調整する

for trial in range(1000):   # 訓練回数

    rnd = np.random.randint(0, N_DATA)  # ランダムに1個データを取り出す
    x = train_x[rnd]
    y = train_y[rnd]
    
    u = np.sum( x * W ) + B
    result = 1 / (1 + np.exp(-u) )  # シグモイドニューロンの実行
    
    error = result - y  # 誤差
    error *= result * (1 - result)  # 誤差にシグモイドの微分を組み入れる(★)
    
    W -= ( LEARNING_RATE * error * x )  # 重みの更新
    B -= ( LEARNING_RATE * error )      # バイアスの更新

# 学習の結果、得られた重みとバイアスの適正値
print( "y = {} x + {}".format(W, B) )
